// VISAterminalDlg.cpp : implementation file
//

#include "stdafx.h"

#include "ConnectDialog.h"
#include "..\VisaDev.h"

#include "VISAterminal.h"
#include "VISAterminalDlg.h"
#include ".\visaterminaldlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CVISAterminalDlg dialog



CVISAterminalDlg::CVISAterminalDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CVISAterminalDlg::IDD, pParent)
	, m_sConnectStatus(_T(""))
	, m_sCommand(_T(""))
	, m_sReply(_T(""))
	, m_sErrorQueue(_T(""))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	
	m_sVisaGenerator = AfxGetApp()->GetProfileString("", "VisaGenerator", "28");
	m_pSMx = NULL;
	m_bIsConnected = false;
}

CVISAterminalDlg::~CVISAterminalDlg()
{
    ExitMeasurement();

}
void CVISAterminalDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_CONNECT_STATUS, m_sConnectStatus);
	DDX_Text(pDX, IDC_EDIT1, m_sCommand);
	DDX_Text(pDX, IDC_REPLY, m_sReply);
	DDX_Text(pDX, IDC_ERROR_QUEUE, m_sErrorQueue);
}

BEGIN_MESSAGE_MAP(CVISAterminalDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_CONNECT, OnBnClickedConnect)
	ON_BN_CLICKED(IDC_SEND, OnBnClickedSend)
	ON_BN_CLICKED(IDC_IDN, OnBnClickedIdn)
	ON_BN_CLICKED(IDC_RST, OnBnClickedRst)
	ON_BN_CLICKED(IDC_OPT, OnBnClickedOpt)
	ON_BN_CLICKED(IDC_OUTP, OnBnClickedOutp)
END_MESSAGE_MAP()


// CVISAterminalDlg message handlers

BOOL CVISAterminalDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	InitMeasurement();
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CVISAterminalDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CVISAterminalDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CVISAterminalDlg::OnBnClickedConnect()
{
	// TODO: Add your control notification handler code here
	//initiate for selecting VISA interfaces
	InitMeasurement();
}

void CVISAterminalDlg::InitMeasurement(void)
{
    if (m_pSMx)
    {
        delete m_pSMx;
        m_pSMx = NULL;
    }
	
	//prompt for VISA address
	CConnectDialog myDialog(this);

	//copy last strings to dialog
	myDialog.m_sVisaGenerator = m_sVisaGenerator;
	
	if (myDialog.DoModal() == IDOK)
	{
		//get back the strings
		m_sVisaGenerator = myDialog.m_sVisaGenerator;
	}
	
	//Smx Connect
	m_pSMx = new CVISADev(m_sVisaGenerator, false);

	//successful?
	if (m_pSMx->IsDeviceReady())
	{
		m_sConnectStatus = m_pSMx->Request("*IDN?");
		m_pSMx->Write("*CLS");
		m_bIsConnected = true;
	}
	else
	{
		m_sConnectStatus = "No generator found on "+m_sVisaGenerator+"!";
	}

	UpdateData(false);
}


void CVISAterminalDlg::ExitMeasurement(void)
{
    if (m_pSMx)
    {
        delete m_pSMx;
        m_pSMx = NULL;
    }
	
    m_bIsConnected = false;

	//Werte speichern 
	AfxGetApp()->WriteProfileString("", "VisaGenerator", m_sVisaGenerator);
	
}

void CVISAterminalDlg::OnBnClickedSend()
{
	// TODO: Add your control notification handler code here
	if (!m_bIsConnected)
		InitMeasurement();

	UpdateData(true);

	if (m_sCommand.Find("?") > -1 )
		m_sReply = m_pSMx->Request(m_sCommand);
	else
		m_pSMx->Write(m_sCommand);

	m_sErrorQueue = m_pSMx->Request("SYST:ERR?");
	UpdateData(false);

}

void CVISAterminalDlg::OnBnClickedIdn()
{
	// TODO: Add your control notification handler code here
	m_sReply = m_pSMx->Request("*IDN?");
	m_sErrorQueue = m_pSMx->Request("SYST:ERR?");
	UpdateData(false);
}

void CVISAterminalDlg::OnBnClickedRst()
{
	// TODO: Add your control notification handler code here
	m_pSMx->Write("*RST");
	m_sErrorQueue = m_pSMx->Request("SYST:ERR?");
	UpdateData(false);
}

void CVISAterminalDlg::OnBnClickedOpt()
{
	// TODO: Add your control notification handler code here
	m_sReply = m_pSMx->Request("*OPT?");
	m_sErrorQueue = m_pSMx->Request("SYST:ERR?");
	UpdateData(false);
}

void CVISAterminalDlg::OnBnClickedOutp()
{
	// TODO: Add your control notification handler code here
	m_pSMx->Write("FREQ 1GHZ");
	m_pSMx->Write("POW 0DBM");
	m_pSMx->Write("OUTP:STAT ON");
	
	m_sErrorQueue = m_pSMx->Request("SYST:ERR?");
	UpdateData(false);
}
