#include <QFile>
#include <QString>
#include "TargetDirBrowser.h"
#include "ui_TargetDirBrowser.h"
#include "treemodel.h"
#include "SCPIsocket.h"

#define MAX_BUFFER_SIZE 10000
#define READ_PACKET_SIZE 1000


TargetDirBrowser::TargetDirBrowser(SCPIsocket *targetSocket, QString topDirectory, QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::TargetDirBrowser)
{
    m_ui->setupUi(this);
    m_targetSocket = targetSocket;

    //Initialize the recursive function call
    m_topDirectory = topDirectory;
    directoryData = "";

    fillDirectoryData(0);
    pModel = new TreeModel(directoryData);

    m_ui->treeView->setModel(pModel);

    m_ui->treeView->resizeColumnToContents(0);

}

//fill all the directory in one single string
void TargetDirBrowser::fillDirectoryData(int level)
{

    //Debugoutput
    //qDebug("Level = %d, Directory = ", level);
    //qDebug(m_topDirectory.toLatin1());

    //Set the directory
    m_targetSocket->   write("MMEM:CDIR '" + m_topDirectory);

    //if (!parent->readErrorQueue(false))
    {

        m_targetSocket->write("MMEM:CAT?\n");

        //The reply will be very fast, so we can afford to read synchonously
        char data[MAX_BUFFER_SIZE];

        //However, we have to spend some effort because the data may be sent in blocks
        qint64 len = 0L, totalLen = 0L;

        do
        {
            len = m_targetSocket->read(data+totalLen, READ_PACKET_SIZE);
            if (len > 0)
                totalLen += len;

        }
        while((len > 0) && (totalLen < MAX_BUFFER_SIZE-READ_PACKET_SIZE));

        //Anything received?
        if (totalLen)
        {
            //qDebug(data);

            //filter all directories out
            QString totalCatalog(data);
            QString fileInCatalog;

            int dirCount = 1;
            do
            {
                //Outer Loop : scan to file descriptors
                fileInCatalog = totalCatalog.section('\"', dirCount, dirCount);
                dirCount+=2;

                //is this file a directory?
                if (fileInCatalog.section(',', 1, 1) == "DIR")
                {
                    //separate first part
                    fileInCatalog = fileInCatalog.section(',', 0, 0);

                    //is it a real directory or "." or ".."
                    if ((fileInCatalog != "..") && (fileInCatalog != "."))
                    {
                        //ok, this is worth to deal with...
                        //four spaces indent per level
                        int i;
                        for (i=0;i<level;i++)
                            directoryData += "    ";

                        //set current directory as top directory but save current directory first
                        QString saveDirectory = m_topDirectory;
                        m_topDirectory += "\\" + fileInCatalog;

                        //store this directory twice
                        //first column : name without context
                        directoryData += fileInCatalog + "\t";
                        //second column : name with full context
                        directoryData += m_topDirectory + "\n";

                        //recursive call
                        fillDirectoryData(level + 1);

                        //restore directory;
                        m_topDirectory = saveDirectory;
                    }
                }
            }
            while (!fileInCatalog.isEmpty());
        }
        else
            qDebug("Socket Read Error" );
    } //if no entry in error queue


}

TargetDirBrowser::~TargetDirBrowser()
{
    delete m_ui;
    delete pModel;
}

void TargetDirBrowser::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}


void TargetDirBrowser::clicked(const QModelIndex & index)
{
    //patch column to 1 to get full qualified path
    QVariant variant = pModel->data(index.sibling(index.row(), 1), Qt::DisplayRole);
    targetDir = variant.toString();
    qDebug(targetDir.toLatin1());

}
