#include <stdlib.h>
#include <QFile>
#include <QFileDialog>
#include <QMessageBox>
#include <QProgressDialog>
#include <QStringList>
#include "crawethernetchanneltest.h"
#include "ui_crawethernetchanneltest.h"
#include "ctcpsocket.h"
#include "cinisettings.h"

CRawEthernetChannelTest::CRawEthernetChannelTest(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::Dialog_RawEthernetChannelTest)
{
  ui->setupUi(this);

  CIniSettings iniSettings;
  QString qsHostname = iniSettings.value("RawEthernetChannelDialog/Hostname", "" ).toString();
  QString qsPort = iniSettings.value("RawEthernetChannelDialog/Port", "5025" ).toString();
  QString qsTimeout = iniSettings.value("RawEthernetChannelDialog/Timeout", "10" ).toString();
  QString qsIteration = iniSettings.value("RawEthernetChannelDialog/Iteration", "10" ).toString();
  int iEnhancedTestState = iniSettings.value("RawEthernetChannelDialog/EnhancedTestState", 0 ).toInt();
  int iHoldConnectionState = iniSettings.value("RawEthernetChannelDialog/HoldConnectionState", 0 ).toInt();
  QString qsWaveformFilename = iniSettings.value("RawEthernetChannelDialog/WaveformFilename", "" ).toString();

  ui->lineEdit_Hostname->setText(qsHostname);
  ui->lineEdit_Port->setText(qsPort);
  ui->lineEdit_Timeout->setText(qsTimeout);
  ui->lineEdit_Iteration->setText(qsIteration);

  if ( iEnhancedTestState == Qt::Unchecked )
  {
    ui->checkBox_EnhancedTests->setCheckState(Qt::Unchecked);
    ui->checkBox_HoldConnection->setEnabled(false);
    ui->pushButton_SelectWaveform->setEnabled(false);
    ui->lineEdit_Waveform->setEnabled(false);
  }
  else if ( iEnhancedTestState == Qt::Checked )
  {
    ui->checkBox_EnhancedTests->setCheckState(Qt::Checked);
    ui->checkBox_HoldConnection->setEnabled(true);
    ui->pushButton_SelectWaveform->setEnabled(true);
    ui->lineEdit_Waveform->setEnabled(true);
    if ( iHoldConnectionState == Qt::Unchecked )
      ui->checkBox_HoldConnection->setCheckState(Qt::Unchecked);
    else if ( iHoldConnectionState == Qt::Checked )
      ui->checkBox_HoldConnection->setCheckState(Qt::Checked);

    ui->lineEdit_Waveform->setText(qsWaveformFilename);
  }


  connect(ui->pushButton_Close, SIGNAL(released()), this, SLOT(close()));
  connect(ui->pushButton_Execute, SIGNAL(released()), this, SLOT(execute()));
  connect(ui->checkBox_EnhancedTests, SIGNAL(stateChanged(int)), this, SLOT(enhancedTest(int)));
  connect(ui->pushButton_SelectWaveform, SIGNAL(released()), this, SLOT(selectWaveform()));
}

CRawEthernetChannelTest::~CRawEthernetChannelTest()
{
  delete ui;
  ui = NULL;
}

void CRawEthernetChannelTest::close()
{
  QString qsHostname = ui->lineEdit_Hostname->text();
  QString qsPort = ui->lineEdit_Port->text();
  QString qsTimeout = ui->lineEdit_Timeout->text();
  QString qsIteration = ui->lineEdit_Iteration->text();
  int iEnhancedTestState = ui->checkBox_EnhancedTests->checkState();
  int iHoldConnectionState = ui->checkBox_HoldConnection->checkState();
  QString qsWaveformFilename = ui->lineEdit_Waveform->text();

  CIniSettings iniSettings;
  iniSettings.setValue( "RawEthernetChannelDialog/Hostname", qsHostname );
  iniSettings.setValue( "RawEthernetChannelDialog/Port", qsPort );
  iniSettings.setValue( "RawEthernetChannelDialog/Timeout", qsTimeout );
  iniSettings.setValue( "RawEthernetChannelDialog/Iteration", qsIteration );
  iniSettings.setValue( "RawEthernetChannelDialog/EnhancedTestState", iEnhancedTestState );
  iniSettings.setValue( "RawEthernetChannelDialog/HoldConnectionState", iHoldConnectionState );
  iniSettings.setValue( "RawEthernetChannelDialog/WaveformFilename", qsWaveformFilename );

  QDialog::close();
}

void CRawEthernetChannelTest::execute()
{
  bool result = true;
  QString qsHostname = ui->lineEdit_Hostname->text();
  QString qsPort = ui->lineEdit_Port->text();
  QString qsTimeout = ui->lineEdit_Timeout->text();
  QString qsIteration = ui->lineEdit_Iteration->text();
  int iIteration = qsIteration.toInt();
  int iEnhancedTestState = ui->checkBox_EnhancedTests->checkState();
  int iHoldConnectionState = ui->checkBox_HoldConnection->checkState();

  QString qsResult;
  ui->textEdit_Result->clear();
  _sleep(1);

  if ( iEnhancedTestState == Qt::Unchecked )
  {
    //check parameter
    // to do
    QProgressDialog progress("Running Raw Ethernet Channel Test", QString(), 0, iIteration, this );
    progress.setWindowModality(Qt::WindowModal);

    for ( int i = 0; i < iIteration; i++ )
    {
      progress.setValue(i);
      int error = 0;
      CTcpSocket *pTcpSocket = new CTcpSocket;
      pTcpSocket->setTimeout(qsTimeout.toInt());
      pTcpSocket->showInfoMsg(false);

      error = pTcpSocket->connect2Server(qsHostname, qsPort.toInt());
      if ( error == 0 )
      {
        // read *idn?
        QByteArray qWriteByteArray("*idn?");
        pTcpSocket->write(qWriteByteArray);
        QByteArray qReadByteArray;
        pTcpSocket->read(qReadByteArray);

        // do preset
        qWriteByteArray = "*rst";
        pTcpSocket->write(qWriteByteArray);
        qWriteByteArray = "*opc?";
        pTcpSocket->write(qWriteByteArray);
        pTcpSocket->read(qReadByteArray);

        // set freq and read back
        qWriteByteArray = "FREQ 2000000000.0";
        pTcpSocket->write(qWriteByteArray);
        qWriteByteArray = "*opc?";
        pTcpSocket->write(qWriteByteArray);
        pTcpSocket->read(qReadByteArray);
        qWriteByteArray = "FREQ?";
        pTcpSocket->write(qWriteByteArray);
        pTcpSocket->read(qReadByteArray);

        // set Level and read back
        qWriteByteArray = "POW 0.00dbm";
        pTcpSocket->write(qWriteByteArray);
        qWriteByteArray = "*opc?";
        pTcpSocket->write(qWriteByteArray);
        pTcpSocket->read(qReadByteArray);
        qWriteByteArray = "POW?";
        pTcpSocket->write(qWriteByteArray);
        pTcpSocket->read(qReadByteArray);

        // do preset
        qWriteByteArray = "*rst";
        pTcpSocket->write(qWriteByteArray);
        qWriteByteArray = "*opc?";
        pTcpSocket->write(qWriteByteArray);
        pTcpSocket->read(qReadByteArray);

        error = pTcpSocket->disconnect();
      }

      delete pTcpSocket;
      pTcpSocket = NULL;
    }

    progress.setValue(iIteration);
  }
  else if ( iEnhancedTestState == Qt::Checked )
  {
    QString qsWaveformFilename = ui->lineEdit_Waveform->text();
    QFile qWaveformFile(qsWaveformFilename);

    if ( qWaveformFile.exists() )
    {
      if ( qWaveformFile.open( QIODevice::ReadOnly ) )
      {
        int iFileSize = qWaveformFile.size();
        QByteArray qMMemCmdString( "MMEM:DATA '/hdd/share/temp.wv',#" + QByteArray().setNum((QByteArray().setNum(iFileSize)).size()) + QByteArray().setNum(iFileSize) );
        QByteArray qDataArray = qWaveformFile.readAll();
        qMMemCmdString += qDataArray;

        if ( iHoldConnectionState == Qt::Unchecked )
        {
          QProgressDialog progress("Running Raw Ethernet Channel Test", QString(), 0, iIteration, this );
          progress.setWindowModality(Qt::WindowModal);
          QByteArray qReadByteArray;
          int error = 0;

          for ( int i = 0; (i < iIteration && error == 0); i++ )
          {
            progress.setValue(i);
            CTcpSocket *pTcpSocket = new CTcpSocket;
            pTcpSocket->setTimeout(qsTimeout.toInt());
            pTcpSocket->showInfoMsg(false);

            error = pTcpSocket->connect2Server(qsHostname, qsPort.toInt());
            _sleep(1);
            if ( error == 0 )
            {
              qReadByteArray.clear();
              error = pTcpSocket->write(qMMemCmdString);
              _sleep(1);
              if ( error == 0 )
              {
                qsResult.append( QString().setNum(i) + QString(": Data written\n") );
                ui->textEdit_Result->setText(qsResult);
              }
              else
              {
                qsResult.append( QString().setNum(i) + QString(": Error while writing data\n") );
                ui->textEdit_Result->setText(qsResult);
              }

              QByteArray qWriteByteArray = "*opc?";
              error = pTcpSocket->write(qWriteByteArray);
              error = pTcpSocket->read(qReadByteArray);
              if ( error == 0 )
              {
                qsResult.append( QString().setNum(i) + QString(": *opc? ready\n"));
                ui->textEdit_Result->setText(qsResult);
              }
              else
              {
                qsResult.append( QString().setNum(i) + QString(": Error while reading data\n") );
                ui->textEdit_Result->setText(qsResult);
              }
              qWriteByteArray = "SYST:ERR?";
              error = pTcpSocket->write(qWriteByteArray);
              error = pTcpSocket->read(qReadByteArray);
              _sleep(1);
              if ( error == 0 )
              {
                qsResult.append( QString().setNum(i) + QString(": SYST:ERR? ready -- ") + qReadByteArray + QString("\n") );
                ui->textEdit_Result->setText(qsResult);
              }
              else
              {
                qsResult.append( QString().setNum(i) + QString(": Error while reading data\n") );
                ui->textEdit_Result->setText(qsResult);
              }

              QString qsSysErr = qReadByteArray;

              error = pTcpSocket->disconnect();
              _sleep(1);
            }
            delete pTcpSocket;
            pTcpSocket = NULL;
            progress.setValue(iIteration);
          }
        }
        else if ( iHoldConnectionState == Qt::Checked )
        {
          QProgressDialog progress("Running Raw Ethernet Channel Test", QString(), 0, iIteration, this );
          progress.setWindowModality(Qt::WindowModal);
          QByteArray qReadByteArray;
          int error = 0;

          CTcpSocket *pTcpSocket = new CTcpSocket;
          pTcpSocket->setTimeout(qsTimeout.toInt());
          pTcpSocket->showInfoMsg(false);

          error = pTcpSocket->connect2Server(qsHostname, qsPort.toInt());
          _sleep(1);
          if ( error == 0 )
          {
            for ( int i = 0; (i < iIteration && error == 0); i++ )
            {
              progress.setValue(i);
              qReadByteArray.clear();
              error = pTcpSocket->write(qMMemCmdString);
              _sleep(1);
              if ( error == 0 )
              {
                qsResult.append( QString().setNum(i) + QString(": Data written\n") );
                ui->textEdit_Result->setText(qsResult);
              }
              else
              {
                qsResult.append( QString().setNum(i) + QString(": Error while writing data\n") );
                ui->textEdit_Result->setText(qsResult);
              }

              QByteArray qWriteByteArray = "*opc?";
              error = pTcpSocket->write(qWriteByteArray);
              error = pTcpSocket->read(qReadByteArray);
              _sleep(1);
              if ( error == 0 )
              {
                qsResult.append( QString().setNum(i) + QString(": *opc? ready\n"));
                ui->textEdit_Result->setText(qsResult);
              }
              else
              {
                qsResult.append( QString().setNum(i) + QString(": Error while reading data\n") );
                ui->textEdit_Result->setText(qsResult);
              }
              qWriteByteArray = "SYST:ERR?";
              error = pTcpSocket->write(qWriteByteArray);
              error = pTcpSocket->read(qReadByteArray);
              _sleep(1);
              if ( error == 0 )
              {
                qsResult.append( QString().setNum(i) + QString(": SYST:ERR? ready -- ") + qReadByteArray + QString("\n") );
                ui->textEdit_Result->setText(qsResult);
              }
              else
              {
                qsResult.append( QString().setNum(i) + QString(": Error while reading data\n") );
                ui->textEdit_Result->setText(qsResult);
              }

              QString qsSysErr = qReadByteArray;

              _sleep(1);
            }

            progress.setValue(iIteration);

            if ( error == 0 )
            {
              QMessageBox::information(NULL, "Raw Ethernet Channel Test ...", "Raw Ethernet Channel Test finished successfully!");
            }
            else
            {
              QMessageBox::critical(NULL, "Raw Ethernet Channel Test ...", "Raw Ethernet Channel Test failed!\nFor details see result window!" );
            }

            error = pTcpSocket->disconnect();
            delete pTcpSocket;
            pTcpSocket = NULL;
          }
        }
      }
      else
      {
        // could not open file
      }
    }
    else
    {
      // file does not exist
    }
  }
}

void CRawEthernetChannelTest::enhancedTest(int state)
{
  if ( state == Qt::Unchecked )
  {
    ui->checkBox_HoldConnection->setEnabled(false);
    ui->pushButton_SelectWaveform->setEnabled(false);
    ui->lineEdit_Waveform->setEnabled(false);
  }
  else if ( state == Qt::Checked )
  {
    ui->checkBox_HoldConnection->setEnabled(true);
    ui->pushButton_SelectWaveform->setEnabled(true);
    ui->lineEdit_Waveform->setEnabled(true);
  }
}

void CRawEthernetChannelTest::selectWaveform()
{
  QFileDialog qFileDialog(this);
  qFileDialog.setFileMode(QFileDialog::AnyFile);
  qFileDialog.setNameFilter(tr("Waveforms (*.wv)"));
  qFileDialog.setViewMode(QFileDialog::Detail);
  QStringList qFileNames;
  if ( qFileDialog.exec() )
  {
    qFileNames = qFileDialog.selectedFiles();
    ui->lineEdit_Waveform->setText( qFileNames.first() );
  }
}
