#include <QMessageBox>
#include "ctcpsocket.h"

CTcpSocket::CTcpSocket()
: m_pTcpSocket(new QTcpSocket())
, m_iTimeout(10) //seconds
, m_bShowInfoMsg(true)
{
}

CTcpSocket::~CTcpSocket()
{
  delete m_pTcpSocket;
}

int CTcpSocket::connect2Server( QString &hostname, int port)
{
  int error = 0;
  m_pTcpSocket->connectToHost( hostname, port );
  if ( m_pTcpSocket->waitForConnected(m_iTimeout * 1000) )
  {
    if ( m_bShowInfoMsg )
      QMessageBox::information(NULL, "Connecting instrument ...", "Interface opened successfully!");
  }
  else
  {
    error = m_pTcpSocket->error();
    QString qsError = m_pTcpSocket->errorString();
    QString qsErrorString = "Could not initialize interface!\n";
    qsErrorString += QString("Error(") + QString().setNum(error) + QString("): ") + qsError + QString("\n");
    QMessageBox::critical(NULL, "Connecting instrument ...", qsErrorString );
  }
  return( error );
}

int CTcpSocket::disconnect()
{
  int error = 0;
  m_pTcpSocket->disconnectFromHost();
  if ( m_pTcpSocket->state() == QAbstractSocket::UnconnectedState || m_pTcpSocket->waitForDisconnected(m_iTimeout * 1000) )
  {
    if ( m_bShowInfoMsg )
      QMessageBox::information(NULL, "Disconnecting instrument ...", "Interface closed successfully!");
  }
  else
  {
    error = m_pTcpSocket->error();
    QString qsError = m_pTcpSocket->errorString();
    QString qsErrorString = "Could not close interface!\n";
    qsErrorString += QString("Error(") + QString().setNum(error) + QString("): ") + qsError + QString("\n");
    QMessageBox::critical(NULL, "Disconnecting instrument ...", qsErrorString );
  }

  return (error);
}

int CTcpSocket::write( QByteArray &txByteArray )
{
  int error = 0;
  txByteArray.append("\n");
  if ( m_pTcpSocket->isValid() )
  {
    m_pTcpSocket->write(txByteArray);
    if ( !m_pTcpSocket->waitForBytesWritten( m_iTimeout * 1000 ) )
    {
      error = m_pTcpSocket->error();
      QString qsError = m_pTcpSocket->errorString();
      QString qsErrorString = "Could not write command!\n";
      qsErrorString += QString("Error(") + QString().setNum(error) + QString("): ") + qsError + QString("\n");
      QMessageBox::critical(NULL, "Writing to instrument ...", qsErrorString );
    }
    int bytes2Write = m_pTcpSocket->bytesToWrite();
  }
  else
  {
    error = m_pTcpSocket->error();
    QString qsError = m_pTcpSocket->errorString();
    QString qsErrorString = "Could not write command!\n";
    qsErrorString += QString("Error(") + QString().setNum(error) + QString("): ") + qsError + QString("\n");
    QMessageBox::critical(NULL, "Writing to instrument ...", qsErrorString );
  }
  return(error);
}

int CTcpSocket::read( QByteArray &rxByteArray )
{
  int error = 0;
  rxByteArray.clear();
  if ( m_pTcpSocket->isValid() )
  {
    int byteAvail = m_pTcpSocket->bytesAvailable();
    if ( !m_pTcpSocket->waitForReadyRead( m_iTimeout *1000 ) )
    {
      error = m_pTcpSocket->error();
      QString qsError = m_pTcpSocket->errorString();
      QString qsErrorString = "Could not read data!\n";
      qsErrorString += QString("Error(") + QString().setNum(error) + QString("): ") + qsError + QString("\n");
      QMessageBox::critical(NULL, "Reading from instrument ...", qsErrorString );
    }
    byteAvail = m_pTcpSocket->bytesAvailable();
    rxByteArray = m_pTcpSocket->readAll();
  }
  else
  {
    error = m_pTcpSocket->error();
    QString qsError = m_pTcpSocket->errorString();
    QString qsErrorString = "Could not read data!\n";
    qsErrorString += QString("Error(") + QString().setNum(error) + QString("): ") + qsError + QString("\n");
    QMessageBox::critical(NULL, "Reading from instrument ...", qsErrorString );
  }
  return(error);
}

QString CTcpSocket::getCurrentHostname()
{
  return( m_pTcpSocket->peerName() );
}

int CTcpSocket::getCurrentPort()
{
  return( m_pTcpSocket->peerPort() );
}

void CTcpSocket::setTimeout( int iTimeout )
{
  m_iTimeout = iTimeout;
}

void CTcpSocket::showInfoMsg(bool bShowInfoMsg)
{
  m_bShowInfoMsg = bShowInfoMsg;
}
